/*
*********************************************************************************************************
*
*                                             EXAMPLE CODE
*
*                          (c) Copyright 2003-2007; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*
*               Knowledge of the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                             EXAMPLE CODE
*
* Filename      : app.c
* Version       : V1.89
* Programmer(s) : Jean-Denis Hatier
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#include  <includes.h>

#if APP_TCPIP_EN
#include  <net.h>
#endif

#if APP_VIEW_EN
#include  <os_viewc.h>
#include  <os_view.h>
#endif

#if APP_CLK_EN
#include  <clk.h>
#endif

#if APP_FS_EN
#include  <FS_API.h>
#endif

#if APP_DHCPc_EN
#include  <dhcp-c.h>
#endif

#if APP_DNSc_EN
#include  <dns-c.h>
#endif

#if APP_FTPc_EN
#include  <ftp-c.h>
#endif

#if APP_FTPs_EN
#include  <ftp-s.h>
#endif

#if APP_HTTPs_EN
#include  <http-s.h>
#endif

#if APP_POP3c_EN
#include  <pop3-c.h>
#endif

#if APP_SMTPc_EN
#include  <smtp-c.h>
#endif

#if APP_SNTPc_EN
#include  <sntp-c.h>
#endif

#if APP_TFTPs_EN
#include  <tftp-s.h>
#endif

#if APP_TTCP_EN
#include  <ttcp.h>
#endif


/*
*********************************************************************************************************
*                                               VARIABLES
*********************************************************************************************************
*/

static  OS_STK          App_StartTaskStk[APP_START_OS_CFG_TASK_STK_SIZE];
static  OS_STK          App_1_TaskStk[APP_1_OS_CFG_TASK_STK_SIZE];

#if APP_TCPIP_EN
static  NET_IP_ADDR     App_IP_Addr;
static  NET_IP_ADDR     App_IP_Mask;
static  NET_IP_ADDR     App_IP_DfltGateway;
static  NET_IP_ADDR     App_IP_DNS_Srvr;
static  NET_IP_ADDR     App_IP_NTP_Srvr;
#endif  /* APP_TCPIP_EN */

static  CPU_INT32S      App_Clk_UTC_Offset;

static  CPU_CHAR        App_HostName[CPU_CFG_NAME_SIZE];

static  CPU_INT32U      App_DHCP_LeaseTime;

static  CPU_INT08S      OS_CPU_UsageMax;


/*
*********************************************************************************************************
*                                          FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void       App_TaskCreate      (void);
static  void       App_TaskStart       (void        *p_arg);
static  void       App_Task_1          (void        *p_arg);

#if APP_TCPIP_EN
static  void       App_InitTCPIP       (void);
#endif  /* APP_TCPIP_EN */

#if APP_FS_EN
static  void       App_InitFS          (void);
#endif  /* APP_FS_EN */

#if APP_DHCPc_EN
static  void       App_InitDHCPc       (void);
static  void       DHCPc_Print         (DHCP_HDR    *d);
static  void       DHCPc_PrintOptions  (CPU_INT08U  *options);
static  CPU_CHAR  *DHCPc_GetOperStr    (CPU_INT32U   op);
static  CPU_CHAR  *DHCPc_GetOptStr     (CPU_INT32U   op);
#endif  /* APP_DHCPc_EN */

#if APP_DNSc_EN
static  void       App_TestDNSc        (void);
#endif  /* APP_DNSc_EN */

#if APP_FTPc_EN
static  void       App_TestFTPc        (void);
#endif  /* APP_FTPc_EN */

#if APP_POP3c_EN
static  void       App_TestPOP3c       (void);
#endif  /* APP_POP3c_EN */

#if APP_SMTPc_EN
static  void       App_TestSMTPc       (void);
#endif  /* APP_SMTPc_EN */

#if APP_SNTPc_EN
static  void       App_TestSNTPc       (void);
#endif  /* APP_SNTPc_EN */


/*
*********************************************************************************************************
*                                             C ENTRY POINT
*********************************************************************************************************
*/

int  main (void)
{
#if (OS_TASK_NAME_SIZE >= 16)
    CPU_INT08U  os_err;
#endif  /* (OS_TASK_NAME_SIZE >= 16) */


   (void)&App_Clk_UTC_Offset;
   (void)&App_DHCP_LeaseTime;

    os_err = 0;                                                 /* Warning: With some debuggers the first call is       */
                                                                /* ignored.                                             */

    BSP_Init();                                                 /* Initialize BSP.                                      */
    CPU_Init();                                                 /* Initialize CPU host name.                            */

    APP_TRACE_DEBUG(("\n\n\n"));
    APP_TRACE_DEBUG(("Initialize OS...\n"));
    OSInit();                                                   /* Initialize OS.                                       */

                                                                /* Create start task.                                   */
    OSTaskCreateExt( App_TaskStart,
                    (void *)0,
                    (OS_STK *)&App_StartTaskStk[APP_START_OS_CFG_TASK_STK_SIZE - 1],
                     APP_START_OS_CFG_TASK_PRIO,
                     APP_START_OS_CFG_TASK_PRIO,
                    (OS_STK *)&App_StartTaskStk[0],
                     APP_START_OS_CFG_TASK_STK_SIZE,
                    (void *)0,
                     OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR);

                                                                /* Give a name to tasks.                                */
#if (OS_TASK_NAME_SIZE >= 16)
    OSTaskNameSet(OS_TASK_IDLE_PRIO,          "Idle",  &os_err);
#if (OS_TASK_STAT_EN > 0)
    OSTaskNameSet(OS_TASK_STAT_PRIO,          "Stat",  &os_err);
#endif  /* (OS_TASK_STAT_EN > 0) */
    OSTaskNameSet(APP_START_OS_CFG_TASK_PRIO, "Start", &os_err);
#endif  /* (OS_TASK_NAME_SIZE >= 16) */

    APP_TRACE_DEBUG(("Start OS...\n"));
    OSStart();                                                  /* Start OS.                                            */
}


/*
*********************************************************************************************************
*                                             STARTUP TASK
*
* Description : This is an example of a startup task.  As mentioned in the book's text, you MUST
*               initialize the ticker only once multitasking has started.
*
* Arguments   : p_arg       argument passed to 'AppTaskStart()' by 'OSTaskCreate()'.
*
* Notes       : 1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                  used.  The compiler should not generate any code for this statement.
*********************************************************************************************************
*/

static  void  App_TaskStart (void  *p_arg)
{
#if (CPU_CFG_NAME_EN == DEF_ENABLED)
    CPU_ERR  cpu_err;
#endif


    (void)&p_arg;                                               /* Prevent compiler warning.                            */

    APP_TRACE_DEBUG(("Initialize OS timer...\n"));
    Tmr_Init();                                                 /* Initialize OS timer.                                 */

#if (OS_TASK_STAT_EN > 0)
    APP_TRACE_DEBUG(("Initialize OS statistic task...\n"));
    OSStatInit();                                               /* Initialize OS statistic task.                        */
#endif  /* (OS_TASK_STAT_EN > 0) */

    APP_TRACE_DEBUG(("Create application task...\n"));
    App_TaskCreate();                                           /* Create application task.                             */

#if APP_FS_EN
    App_InitFS();                                               /* Initialize file system.                              */
#endif  /* APP_FS_EN */

#if APP_TCPIP_EN
    App_InitTCPIP();                                            /* Initialize TCP/IP stack.                             */
#endif  /* APP_TCPIP_EN */

    App_HostName[0] = '\0';

#if APP_DHCPc_EN
    App_InitDHCPc();                                            /* Initialize DHCP client.                              */
#endif  /* APP_DHCPc_EN */

    if (App_HostName[0] == '\0') {
        Mem_Copy(&App_HostName, APP_CFG_DFLT_HOST_NAME, CPU_CFG_NAME_SIZE);
    }

    APP_TRACE_DEBUG(("    Host Name   = %s\n", App_HostName));

#if (CPU_CFG_NAME_EN == DEF_ENABLED)
    CPU_NameSet(App_HostName, &cpu_err);                        /* Set CPU name.                                        */
#endif

#if APP_HTTPs_EN
    APP_TRACE_DEBUG(("Initialize HTTP server...\n"));
    HTTPs_Init();                                               /* Initialize HTTP server.                              */
#endif  /* APP_HTTPs_EN */

#if APP_FTPs_EN
    APP_TRACE_DEBUG(("Initialize FTP server...\n"));
    FTPs_Init(App_IP_Addr, FTPs_CFG_DTP_IPPORT);                /* Initialize FTP server.                               */
#endif  /* APP_FTPs_EN */

#if APP_TFTPs_EN
    APP_TRACE_DEBUG(("Initialize TFTP server...\n"));
    TFTPs_Init();                                               /* Initialize TFTP server.                              */
#endif  /* APP_TFTPs_EN */

#if APP_DNSc_EN
    DNSc_Init(App_IP_DNS_Srvr);                                 /* Initialize DNS client.                               */

    App_TestDNSc();                                             /* Test DNS client.                                     */
#endif  /* APP_DNSc_EN */

#if APP_FTPc_EN
    App_TestFTPc();                                             /* Test FTP client.                                     */
#endif  /* APP_FTPc_EN */

#if APP_POP3c_EN
    App_TestPOP3c();                                            /* Test POP3 client.                                    */
#endif  /* APP_POP3c_EN */

#if APP_SMTPc_EN
    App_TestSMTPc();                                            /* Test SMTP client.                                    */
#endif  /* APP_SMTPc_EN */

#if APP_SNTPc_EN
    App_TestSNTPc();                                            /* Test SNTP client.                                    */
#endif  /* APP_SNTPc_EN */

#if APP_VIEW_EN
    OSView_Init(BSP_DBGU_RATE);
#endif

#if APP_TTCP_EN                                                 /* TTCP contains active user input code. It cannot work */
                                                                /* with other code yep beacuse it takes all the CPU.    */
    APP_TRACE_DEBUG(("\n****************************************************************************"));
    APP_TRACE_DEBUG(("\n*                                                                          *"));
    APP_TRACE_DEBUG(("\n*              Micrium uC/TCP-IP TTCP Performance measurement              *"));
    APP_TRACE_DEBUG(("\n*                     AT91RM9200 on Cogent CSB637 SDK                      *"));
    APP_TRACE_DEBUG(("\n*                                                                          *"));
    APP_TRACE_DEBUG(("\n****************************************************************************"));
    APP_TRACE_DEBUG(("\n"));
    TTCP_Init();                                                /* Initialize TTCP application                          */
#else
    APP_TRACE_DEBUG(("\nSystem ready.\r\n"));
#endif  /* APP_TTCP_EN */

    LED_Off(1);
    LED_Off(2);
    LED_Off(3);

    while (DEF_YES) {                                           /* Task body, always written as an infinite loop.       */
        OSTimeDlyHMSM(0, 0, 0, 500);

#if APP_TFTPs_EN
#if (TFTPs_TRACE_LEVEL >= TRACE_LEVEL_INFO)
        TFTPs_Disp();
        TFTPs_DispTrace();
#endif  /* (TFTPs_TRACE_LEVEL >= TRACE_LEVEL_INFO) */
#endif  /* APP_TFTPs_EN */
    }
}


#if APP_FS_EN
/*
*********************************************************************************************************
*                                        INITIALIZE FILE SYSTEM
*********************************************************************************************************
*/

static  void  App_InitFS (void)
{
    CPU_INT32U  rtn_val;


    APP_TRACE_DEBUG(("Initialize filesystem...\n"));
    FS_Init();
    rtn_val = FS_IoCtl("ram:", FS_CMD_REQUIRES_FORMAT, 0, 0);
    if (rtn_val != 0) {
        APP_TRACE_DEBUG(("Cannot format RAM disk.\n"));
    }
}
#endif  /* APP_FS_EN */


#if APP_TCPIP_EN
/*
*********************************************************************************************************
*                                        INITIALIZE TCP/IP STACK
*********************************************************************************************************
*/

static  void  App_InitTCPIP (void)
{
    NET_ERR      err;
    CPU_INT08U  *pmac;


    APP_TRACE_DEBUG(("Initialize TCP/IP stack...\n"));

    pmac = (CPU_INT08U *)mon_getenv("ETHERADD");
    NetASCII_Str_to_MAC(pmac, NetIF_MAC_Addr, &err);

    err = Net_Init();
    if (err != NET_ERR_NONE) {
        APP_TRACE_DEBUG(("Net_Init() failed: error #%d, line #%d.\n", err, __LINE__));
        while (DEF_YES) {
            ;
        }
    }

#if APP_DHCPc_EN
    APP_TRACE_DEBUG(("    MAC address = %s\n", pmac));
    APP_TRACE_DEBUG(("    IP address  = 0.0.0.0\n"));
    APP_TRACE_DEBUG(("    IP mask     = 0.0.0.0\n"));
    APP_TRACE_DEBUG(("    IP gateway  = 0.0.0.0\n"));

    NetIP_CfgAddrThisHost(NET_IP_ADDR_THIS_HOST, NET_IP_ADDR_NONE);
    NetIP_CfgAddrDfltGateway(NET_IP_ADDR_NONE);

#else  /* APP_DHCPc_EN */
    App_IP_Addr        = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_THIS_HOST,    &err);
    App_IP_Mask        = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_NET_MASK,     &err);
    App_IP_DfltGateway = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_DFLT_GATEWAY, &err);
    App_IP_DNS_Srvr    = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_DNS_SRVR,     &err);
    App_IP_NTP_Srvr    = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_NTP_SRVR,     &err);
    App_Clk_UTC_Offset = APP_CFG_CLK_UTC_OFFSET;

    NetIP_CfgAddrThisHost(App_IP_Addr, App_IP_Mask);
    NetIP_CfgAddrDfltGateway(App_IP_DfltGateway);

    APP_TRACE_DEBUG(("    MAC address = %s\n", pmac));
    APP_TRACE_DEBUG(("    IP address  = %s\n", APP_CFG_IP_ADDR_STR_THIS_HOST));
    APP_TRACE_DEBUG(("    IP mask     = %s\n", APP_CFG_IP_ADDR_STR_NET_MASK));
    APP_TRACE_DEBUG(("    IP gateway  = %s\n", APP_CFG_IP_ADDR_STR_DFLT_GATEWAY));

   (void)&App_IP_DNS_Srvr;
   (void)&App_IP_NTP_Srvr;

#endif  /* APP_DHCPc_EN */
}
#endif  /* APP_TCPIP_EN */


#if APP_DHCPc_EN
/*
*********************************************************************************************************
*                                        INITIALIZE DHCP CLIENT
*********************************************************************************************************
*/

static  void  App_InitDHCPc (void)
{
    NET_ERR      err;
    CPU_INT08U  *opt;
    CPU_INT08U   buf[16];
    void        *param_pos;
    CPU_INT08U   param_len;
    CPU_INT08U   param_req_list[] = {DHCP_OPT_HOST_NAME,
                                     DHCP_OPT_DOMAIN_NAME,
                                     DHCP_OPT_DOMAIN_NAME_SERVER,
                                     DHCP_OPT_NETWORK_TIME_PROTOCOL_SERVER,
                                     DHCP_OPT_TIME_OFFSET};


    DHCPc_SetMacAddr(NetIF_MAC_Addr);
    DHCPc_SetClientID(0x01, NetIF_MAC_Addr, NET_IF_ADDR_SIZE);
    DHCPc_SetVendorClassID("YourCompany", 11);
    DHCPc_SetParamReqList(param_req_list, sizeof(param_req_list) / sizeof(param_req_list[0]));

    APP_TRACE_DEBUG(("Get DHCP lease... "));
    err = DHCPc_Start();
    if (err != DHCPc_ERR_NONE) {
        APP_TRACE_DEBUG(("failed!\n"));
        while (DEF_YES) {
            ;                                                   /* Can't obtain a DHCP lease: nothing to do!            */
        }
    }

    APP_TRACE_DEBUG(("success!\n"));

    DHCPc_CfgStack();

    DHCPc_Print(DHCPc_GetHdr());

                                                                /* Must Mem_Copy because header may be misaliged.       */
    Mem_Copy(&App_IP_Addr,
             &(DHCPc_GetHdr()->yiaddr),
              sizeof(DHCPc_GetHdr()->yiaddr));
    App_IP_Addr            = NET_UTIL_NET_TO_HOST_32(App_IP_Addr);

    opt = DHCPc_GetOpt(DHCP_OPT_SUBNET_MASK);
    if (opt != (void *)0) {
        param_pos = ((void *)(opt + 2));
        param_len = (*(CPU_INT08U *)(opt + 1));
        Mem_Copy(&App_IP_Mask,        param_pos, param_len);
        App_IP_Mask        = NET_UTIL_NET_TO_HOST_32(App_IP_Mask);
    }

    opt = DHCPc_GetOpt(DHCP_OPT_ROUTER);
    if (opt != (void *)0) {
        param_pos = ((void *)(opt + 2));
        param_len = (*(CPU_INT08U *)(opt + 1));
        Mem_Copy(&App_IP_DfltGateway, param_pos, param_len);
        App_IP_DfltGateway = NET_UTIL_NET_TO_HOST_32(App_IP_DfltGateway);
    }

    opt = DHCPc_GetOpt(DHCP_OPT_DOMAIN_NAME_SERVER);
    if (opt != (void *)0) {
        param_pos = ((void *)(opt + 2));
        param_len = (*(CPU_INT08U *)(opt + 1));
        Mem_Copy(&App_IP_DNS_Srvr,    param_pos, param_len);
        App_IP_DNS_Srvr    = NET_UTIL_NET_TO_HOST_32(App_IP_DNS_Srvr);
    } else {
        App_IP_DNS_Srvr    = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_DNS_SRVR, &err);
    }

    opt = DHCPc_GetOpt(DHCP_OPT_NETWORK_TIME_PROTOCOL_SERVER);
    if (opt != (void *)0) {
        param_pos = ((void *)(opt + 2));
        param_len = (*(CPU_INT08U *)(opt + 1));
        Mem_Copy(&App_IP_NTP_Srvr,    param_pos, param_len);
        App_IP_NTP_Srvr    = NET_UTIL_NET_TO_HOST_32(App_IP_NTP_Srvr);
    } else {
        App_IP_NTP_Srvr    = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_NTP_SRVR, &err);
    }

    opt = DHCPc_GetOpt(DHCP_OPT_TIME_OFFSET);
    if (opt != (void *)0) {
        param_pos = ((void *)(opt + 2));
        param_len = (*(CPU_INT08U *)(opt + 1));
        Mem_Copy(&App_Clk_UTC_Offset, param_pos, param_len);
        App_Clk_UTC_Offset = NET_UTIL_NET_TO_HOST_32(App_Clk_UTC_Offset);
    } else {
        App_Clk_UTC_Offset = APP_CFG_CLK_UTC_OFFSET;
    }

    opt = DHCPc_GetOpt(DHCP_OPT_HOST_NAME);
    if (opt != (void *)0) {
        param_pos = ((void *)(opt + 2));
        param_len = (*(CPU_INT08U *)(opt + 1));
        if (param_len < CPU_CFG_NAME_SIZE) {
            Mem_Copy(&App_HostName,   param_pos, param_len);
        } else {
            Mem_Copy(&App_HostName,   param_pos, CPU_CFG_NAME_SIZE);
        }
    }

    opt = DHCPc_GetOpt(DHCP_OPT_IP_ADDRESS_LEASE_TIME);
    if (opt != (void *)0) {
        param_pos = ((void *)(opt + 2));
        param_len = (*(CPU_INT08U *)(opt + 1));
        Mem_Copy(&App_DHCP_LeaseTime, param_pos, param_len);
        App_DHCP_LeaseTime = NET_UTIL_NET_TO_HOST_32(App_DHCP_LeaseTime);
    }

    NetASCII_IP_to_Str(App_IP_Addr,        buf, DEF_NO, &err);
    APP_TRACE_DEBUG(("    DHCP IP address : %s\n", buf));

    NetASCII_IP_to_Str(App_IP_Mask,        buf, DEF_NO, &err);
    APP_TRACE_DEBUG(("    DHCP IP mask    : %s\n", buf));

    NetASCII_IP_to_Str(App_IP_DfltGateway, buf, DEF_NO, &err);
    APP_TRACE_DEBUG(("    DHCP IP gateway : %s\n", buf));

    NetASCII_IP_to_Str(App_IP_DNS_Srvr,    buf, DEF_NO, &err);
    APP_TRACE_DEBUG(("    DHCP DNS server : %s\n", buf));

    NetASCII_IP_to_Str(App_IP_NTP_Srvr,    buf, DEF_NO, &err);
    APP_TRACE_DEBUG(("    DHCP NTP server : %s\n", buf));

    APP_TRACE_DEBUG(("    DHCP UTC offset : %d\n", App_Clk_UTC_Offset));

    if (App_DHCP_LeaseTime > 0) {
        APP_TRACE_DEBUG(("    DHCP Lease Time : %d seconds.\n", App_DHCP_LeaseTime));
    } else {
        APP_TRACE_DEBUG(("    DHCP Lease Time : Infinite\n"));
    }
}
#endif  /* APP_DHCPc_EN */


#if APP_DNSc_EN
/*
*********************************************************************************************************
*                                            TEST DNS CLIENT
*********************************************************************************************************
*/

static  void  App_TestDNSc (void)
{
    CPU_INT32U  dns_test;
    CPU_INT08U  buf[16];
    NET_ERR     err;


    dns_test = DNSc_GetHostByName("192.168.0.3");
    NetASCII_IP_to_Str(NET_UTIL_NET_TO_HOST_32(dns_test), buf, DEF_NO, &err);
    APP_TRACE_DEBUG(("gethostbyname (\"192.168.0.3\") = %s.\n", buf));
    if (dns_test == 0) {
        APP_TRACE_DEBUG(("Error: %s.\n", DNSc_GetLastErrMsg()));
    }

    dns_test = DNSc_GetHostByName("www.jidec.com");
    NetASCII_IP_to_Str(NET_UTIL_NET_TO_HOST_32(dns_test), buf, DEF_NO, &err);
    APP_TRACE_DEBUG(("gethostbyname (\"www.jidec.com\") = %s.\n", buf));
    if (dns_test == 0) {
        APP_TRACE_DEBUG(("Error: %s.\n", DNSc_GetLastErrMsg()));
    }

    dns_test = DNSc_GetHostByName("www.micrium.com");
    NetASCII_IP_to_Str(NET_UTIL_NET_TO_HOST_32(dns_test), buf, DEF_NO, &err);
    APP_TRACE_DEBUG(("gethostbyname (\"www.micrium.com\") = %s.\n", buf));
    if (dns_test == 0) {
        APP_TRACE_DEBUG(("Error: %s.\n", DNSc_GetLastErrMsg()));
    }

    dns_test = DNSc_GetHostByName("345345ssbn5");
    NetASCII_IP_to_Str(NET_UTIL_NET_TO_HOST_32(dns_test), buf, DEF_NO, &err);
    APP_TRACE_DEBUG(("gethostbyname (\"345345ssbn5\") = %s.\n", buf));
    if (dns_test == 0) {
        APP_TRACE_DEBUG(("Error: %s.\n", DNSc_GetLastErrMsg()));
    }
}
#endif  /* APP_DNSc_EN */


#if APP_FTPc_EN
/*
*********************************************************************************************************
*                                            TEST FTP CLIENT
*********************************************************************************************************
*/

#define FTPc_TEST_BUF_SIZE  (2048 * 1024)
static  CPU_INT08U   FTPc_TestBuf[FTPc_TEST_BUF_SIZE];

static  void  App_TestFTPc (void)
{
    NET_IP_ADDR   server_addr;
    CPU_INT16U    server_port;
    CPU_CHAR     *username;
    CPU_CHAR     *password;
    CPU_INT08U   *buf;

    CPU_BOOLEAN   result;
    NET_ERR       net_err;


    APP_TRACE_DEBUG(("Test FTP client...\n"));

    server_addr =  NetASCII_Str_to_IP("192.168.0.2", &net_err);
    server_port =  10210;
    username    = "anonymous";
    password    = "test@test.com";
    buf         = (CPU_INT08U *)0x10000000;

    result = FTPc_Open(server_addr, server_port, username, password, &net_err);
    if (result == DEF_FAIL) {
        APP_TRACE_DEBUG(("FTPc_Open() failed.\n"));
        return;
    }

    APP_TRACE_DEBUG(("FTPc: Sending %d bytes...\n", FTPc_TEST_BUF_SIZE));
    result = FTPc_SendBuf("test/test", buf, FTPc_TEST_BUF_SIZE, DEF_NO, &net_err);
    if (result == DEF_FAIL) {
        APP_TRACE_DEBUG(("FTPc_SendBuf() failed.\n"));
        FTPc_Close(&net_err);
        return;
    }

    APP_TRACE_DEBUG(("FTPc: Receiving %d bytes...\n", FTPc_TEST_BUF_SIZE));
    result = FTPc_RecvFile("test/test", "test", &net_err);
    if (result == DEF_FAIL) {
        APP_TRACE_DEBUG(("FTPc_RecvFile() failed.\n"));
        FTPc_Close(&net_err);
        return;
    }

    APP_TRACE_DEBUG(("FTPc: Sending %d bytes...\n", FTPc_TEST_BUF_SIZE));
    result = FTPc_SendFile("test/test", "test", DEF_NO, &net_err);
    if (result == DEF_FAIL) {
        APP_TRACE_DEBUG(("FTPc_SendFile() failed.\n"));
        FTPc_Close(&net_err);
        return;
    }

    APP_TRACE_DEBUG(("FTPc: Receiving %d bytes...\n", FTPc_TEST_BUF_SIZE));
    result = FTPc_RecvBuf("test /test", FTPc_TestBuf, FTPc_TEST_BUF_SIZE, &net_err);
    if (result == DEF_FAIL) {
        APP_TRACE_DEBUG(("FTPc_RecvBuf() failed.\n"));
        FTPc_Close(&net_err);
        return;
    }

    APP_TRACE_DEBUG(("FTPc: Comparing %d bytes... ", FTPc_TEST_BUF_SIZE));
    result = Mem_Cmp(buf, FTPc_TestBuf, FTPc_TEST_BUF_SIZE);
    if (result == DEF_FAIL) {
        APP_TRACE_DEBUG(("FAILED!\n"));
    } else {
        APP_TRACE_DEBUG(("SUCCESS!\n"));
    }

    result = FTPc_Close(&net_err);
    if (result == DEF_FAIL) {
        APP_TRACE_DEBUG(("FTPc_Close() failed.\n"));
    }

}
#endif  /* APP_FTPc_EN */


#if APP_POP3c_EN

#define  BUFFER_SIZE 2000
#define  POP3c_TEST_SERVER_IP  "0.0.0.0"
#define  POP3c_TEST_USER       ""
#define  POP3c_TEST_PASS       ""

void  App_TestPOP3c (void)
{
    NET_IP_ADDR  ip_server;
    NET_ERR      errMsg;
    NET_SOCK_ID  socket;
    CPU_INT32U   message_qty;
    CPU_INT32U   mailbox_size;
    CPU_INT32U   i;
    CPU_INT32U   j;
    CPU_INT32U   message_size;
    CPU_CHAR     message_buffer[BUFFER_SIZE];
    POP3c_MSG    mail_struct;


    ip_server = NetASCII_Str_to_IP(POP3c_TEST_SERVER_IP, &errMsg);
    if (errMsg != NET_ASCII_ERR_NONE) {
        APP_TRACE_DEBUG(("Error - NetASCII_Str_to_IP: %d\n", errMsg));
        return;
    }

    APP_TRACE_DEBUG(("Connecting...\n"));
    socket = POP3c_Connect(ip_server, 0, &errMsg);
    if (errMsg == POP3c_ERR_NONE) {
        APP_TRACE_DEBUG(("Connected to server (socket: %d)\n", socket));
    } else {
        APP_TRACE_DEBUG(("Error - POP3c_Connect: %d\n", errMsg));
        return;
    }

    POP3c_Authenticate(socket, POP3c_TEST_USER, POP3c_TEST_PASS, &errMsg);
    if (errMsg != POP3c_ERR_NONE) {
        APP_TRACE_DEBUG(("Error - POP3c_Authenticate: %d\n", &errMsg));
        return;
    }


    POP3c_MboxStat(socket, &message_qty, &mailbox_size, &errMsg);
    if (errMsg == POP3c_ERR_NONE) {
        APP_TRACE_DEBUG(("POP3c_MboxStat: %d msg, %d bytes\n", message_qty, mailbox_size));
    } else {
        APP_TRACE_DEBUG(("Error - POP3c_MboxStat: %d\n", errMsg));
    }

    for (i = 1; i <= message_qty; i++)
    {
        POP3c_MsgStat(socket, i, &message_size, &errMsg);
        if (errMsg == POP3c_ERR_NONE) {
            APP_TRACE_DEBUG(("  Message %d: %d bytes\n", i, message_size));
        }
        else {
            APP_TRACE_DEBUG(("  Error - POP3c_MsgStat: %d\n", errMsg));
        }
    }

    for (i = 1; i <= message_qty; i++) {
        APP_TRACE_DEBUG(("\n\n"));

        POP3c_MsgStat(socket, i, &message_size, &errMsg);
        if (errMsg == POP3c_ERR_NONE) {
            APP_TRACE_DEBUG(("  Message %d: %d bytes\n", i, message_size));
        }
        else {
            APP_TRACE_DEBUG(("  Error - POP3c_MsgStat: %d\n", errMsg));
        }

        POP3c_MsgRetrieve(socket, i, message_buffer, BUFFER_SIZE, DEF_NO, &errMsg);
        if (errMsg != POP3c_ERR_NONE) {
            APP_TRACE_DEBUG(("  POP3c_MsgRetrieve error code: %d\n", errMsg));
        }

        POP3c_MsgRead(message_buffer, BUFFER_SIZE, &mail_struct, &errMsg);
        if (errMsg != POP3c_ERR_NONE) {
            APP_TRACE_DEBUG(("  POP3c_MsgRead error code:     %d\n", errMsg));
        }

        APP_TRACE_DEBUG(("\nContent of message %d\n\n", i));
        APP_TRACE_DEBUG(("From     : %s\n",   mail_struct.From));
        APP_TRACE_DEBUG(("Sender   : %s\n",   mail_struct.Sender));
        APP_TRACE_DEBUG(("Date     : %s\n",   mail_struct.Date));
        APP_TRACE_DEBUG(("Reply to : %s\n",   mail_struct.Reply_to));
        APP_TRACE_DEBUG(("Subject  : %s\n\n", mail_struct.Subject));

        APP_TRACE_DEBUG(("%s\n\n", mail_struct.Body));

        APP_TRACE_DEBUG(("Original message (buffer):\n"));

        j = 0;
        while (message_buffer[j] != '\0') {
            APP_TRACE_DEBUG(("%c", message_buffer[j]));
            j++;
        }
    }

    POP3c_Disconnect(socket, &errMsg);
    APP_TRACE_DEBUG(("Disconnected\n"));
}
#endif  /* APP_POP3c_EN */


#if APP_SMTPc_EN
void  App_TestSMTPc (void)
{
    NET_IP_ADDR  ip_server;
    NET_IP_ADDR  client_addr;
    NET_ERR      errMsg;
    NET_SOCK_ID  sock;

    SMTPc_MSG    msg;
    SMTPc_MBOX   from;
    SMTPc_MBOX   to;
    SMTPc_MBOX   cc;
    SMTPc_MBOX   bcc;
    CPU_CHAR     message[] = "Test avec ligne beaucoup trop longue abcdefghijkl "
                             "Test avec ligne beaucoup trop longue abcdefghijkl "
                             "Test avec ligne beaucoup trop longue abcdefghijkl "
                             "Test avec ligne beaucoup trop longue abcdefghijkl "
                             "Plus de 1000";


    ip_server = NetASCII_Str_to_IP("192.168.0.2", &errMsg);
    if (errMsg != NET_ASCII_ERR_NONE) {
        APP_TRACE_DEBUG(( "Error - NetASCII_Str_to_IP: %d\n", errMsg));
        return;
    }
    client_addr = NetASCII_Str_to_IP("192.168.0.11", &errMsg);
    if (errMsg != NET_ASCII_ERR_NONE) {
        APP_TRACE_DEBUG(( "Error - NetASCII_Str_to_IP: %d\n", errMsg));
        return;
    }

    APP_TRACE_DEBUG(("Connecting...\n"));
    sock = SMTPc_Connect(ip_server, 0, client_addr, DEF_NO, &errMsg);
    if (errMsg == SMTPc_ERR_NONE) {
        APP_TRACE_DEBUG(("Connected to server (socket: %d)\n", sock));
    } else {
        APP_TRACE_DEBUG(("Error - SMTPc_Connect: %d\n", errMsg));
        return;
    }

    SMTPc_SetMbox(&from, "J.-D. Hatier", "jdhatier@micrium.com",  &errMsg);
    SMTPc_SetMbox(&to,   "J.-D. Hatier", "jdhatier@micrium.com",  &errMsg);
    SMTPc_SetMbox(&cc,   "J.-D. Hatier", "jdhatier@micrium.com",  &errMsg);
    SMTPc_SetMbox(&bcc,  "J.-D. Hatier", "smtp-test@micrium.com", &errMsg);

    SMTPc_SetMsg(&msg, &from, &from, &errMsg);
    msg.ToArray[0]        = &to;
    msg.CCArray[1]        = &bcc;
    msg.BCCArray[2]       = &cc;
    msg.ContentBodyMsg    = message;
    msg.ContentBodyMsgLen = Str_Len(message);

    Str_Copy(msg.Subject, "Test");

    SMTPc_SendMsg(sock, &msg, &errMsg);

    SMTPc_Disconnect(sock, &errMsg);
    APP_TRACE_DEBUG(("Disconnected\n"));
}
#endif  /* APP_SMTPc_EN */


#if APP_SNTPc_EN
/*
*********************************************************************************************************
*                                            TEST SNTP CLIENT
*********************************************************************************************************
*/

static  void  App_TestSNTPc (void)
{
    CLK_DATE_TIME  date_time;
    CLK_TS_NTP     ntp_time;

    SNTP_PKT       sntp_pkt;
    CPU_INT32U     sntp_test;
    CPU_INT08U     buf[40];


    Clk_Init();

    Clk_DateTime_Make(&date_time, 2006, 1, 1, 0, 0, 0, App_Clk_UTC_Offset);
    Clk_SetDateTime(&date_time);
    Clk_DateTime_ToStr(&date_time, 1, buf);
    APP_TRACE_DEBUG(("CLOCK: %s.\n", buf));

    sntp_test = SNTP_ReqRemoteTime(App_IP_NTP_Srvr, &sntp_pkt);
    if (sntp_test == DEF_OK) {
        ntp_time   = SNTP_GetRemoteTime_s(&sntp_pkt);
        Clk_SetTS_NTP(&ntp_time);
    }

    Clk_GetDateTime(&date_time);
    Clk_DateTime_ToStr(&date_time, 1, buf);
    APP_TRACE_DEBUG(("CLOCK: %s.\n", buf));
    OSTimeDlyHMSM(0, 0, 5, 0);

    Clk_GetDateTime(&date_time);
    Clk_DateTime_ToStr(&date_time, 1, buf);
    APP_TRACE_DEBUG(("CLOCK: %s.\n", buf));
    OSTimeDlyHMSM(0, 0, 5, 0);

    Clk_GetDateTime(&date_time);
    Clk_DateTime_ToStr(&date_time, 1, buf);
    APP_TRACE_DEBUG(("CLOCK: %s.\n", buf));
}


CPU_INT32U  SNTP_GetLocalTime_s (void)
{
    CLK_TS_NTP  ts_ntp;


    Clk_GetTS_NTP(&ts_ntp);
    return ((CPU_INT32U)ts_ntp);
}
#endif  /* APP_SNTPc_EN */


/*
*********************************************************************************************************
*                                       CREATE APPLICATION TASKS
*
* Description : This function creates the application tasks.
*
* Arguments   : None.
*********************************************************************************************************
*/

static  void  App_TaskCreate (void)
{
#if (OS_TASK_NAME_SIZE >= 16)
    CPU_INT08U  err;
#endif  /* (OS_TASK_NAME_SIZE >= 16) */


    OSTaskCreateExt( App_Task_1,
                    (void *)0,
                    (OS_STK *)&App_1_TaskStk[APP_1_OS_CFG_TASK_STK_SIZE - 1],
                     APP_1_OS_CFG_TASK_PRIO,
                     APP_1_OS_CFG_TASK_PRIO,
                    (OS_STK *)&App_1_TaskStk[0],
                     APP_1_OS_CFG_TASK_STK_SIZE,
                    (void *)0,
                     OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR);

#if (OS_TASK_NAME_SIZE >= 16)
    OSTaskNameSet(APP_1_OS_CFG_TASK_PRIO, "Yellow LED task", &err);
#endif  /* (OS_TASK_NAME_SIZE >= 16) */
}


/*
*********************************************************************************************************
*                                                TASK #1
*
* Description : This is an example of a task.
*
* Arguments   : p_arg       argument passed to 'AppTask1()' by 'OSTaskCreate()'.
*
* Notes       : 1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                  used.  The compiler should not generate any code for this statement.
*
*               2) Interrupts are enabled once the task start because the I-bit of the CCR register was
*                  set to 0 by 'OSTaskCreate()'.
*
*               3) This Task uses the LEDs and the switch of the CSB937 board,
*                  it blinks user LED 2 on the CSB937.
*********************************************************************************************************
*/

static  void  App_Task_1 (void  *p_arg)
{
    (void)&p_arg;                                               /* Prevent compiler warning.                            */


    OS_CPU_UsageMax = 0;
    while (DEF_YES) {                                           /* Task body, always written as an infinite loop.       */
        LED_Toggle(1);
        OSTimeDlyHMSM(0, 0, 0, 250);
        if (OS_CPU_UsageMax < OSCPUUsage) {
            OS_CPU_UsageMax = OSCPUUsage;
        }
    }
}


#if APP_DHCPc_EN
/*
*********************************************************************************************************
*                                             DHCPc_Print()
*
* Description : Print the DHCP header...
*********************************************************************************************************
*/

static  void  DHCPc_Print (DHCP_HDR  *d)
{
    CPU_INT08U  *client_ip;
    CPU_INT08U  *your_ip;
    CPU_INT08U  *server_ip;
    CPU_INT08U  *router_ip;
    CPU_INT32U   xid;


    client_ip = (CPU_INT08U *)&(d->ciaddr);
    your_ip   = (CPU_INT08U *)&(d->yiaddr);
    server_ip = (CPU_INT08U *)&(d->siaddr);
    router_ip = (CPU_INT08U *)&(d->giaddr);
    Mem_Copy(&xid, &d->xid, 4);

    APP_TRACE_DEBUG(("    DHCP:\n"));
    APP_TRACE_DEBUG(("        op = %s, htype = %d, hlen = %d, hops = %d\n",
                     DHCPc_GetOperStr(d->op),
                     d->htype,
                     d->hlen,
                     d->hops));
    APP_TRACE_DEBUG(("        seconds = %d, flags = 0x%x, xid= 0x%lx\n",
                     NET_UTIL_NET_TO_HOST_16(d->secs),
                     NET_UTIL_NET_TO_HOST_16(d->flags),
                     NET_UTIL_NET_TO_HOST_32(xid)));
    APP_TRACE_DEBUG(("        caddr = %02x:%02x:%02x:%02x:%02x:%02x\n",
                     d->chaddr[0],
                     d->chaddr[1],
                     d->chaddr[2],
                     d->chaddr[3],
                     d->chaddr[4],
                     d->chaddr[5]));
    APP_TRACE_DEBUG(("        client_ip = %d.%d.%d.%d\n",
                     client_ip[0],
                     client_ip[1],
                     client_ip[2],
                     client_ip[3]));
    APP_TRACE_DEBUG(("        your_ip   = %d.%d.%d.%d\n",
                     your_ip[0],
                     your_ip[1],
                     your_ip[2],
                     your_ip[3]));
    APP_TRACE_DEBUG(("        server_ip = %d.%d.%d.%d\n",
                     server_ip[0],
                     server_ip[1],
                     server_ip[2],
                     server_ip[3]));
    APP_TRACE_DEBUG(("        router_ip = %d.%d.%d.%d\n",
                     router_ip[0],
                     router_ip[1],
                     router_ip[2],
                     router_ip[3]));
    if (d->file[0]  != '\0') {
        APP_TRACE_DEBUG(("        bootfile: %s\n", d->file));
    }
    if (d->sname[0] != '\0') {
        APP_TRACE_DEBUG(("        server_hostname: %s\n", d->sname));
    }

    DHCPc_PrintOptions((CPU_INT08U *)(d + 1));
}


/*
*********************************************************************************************************
*                                        DHCPc_PrintOptions()
*
* Description : Verbosely display the DHCP options pointed to by the incoming options pointer.
*
* Note(s)     : 1) Vendor specific information:
*                  The data within this option is vendor specific.  The RFC2132 says that the encapsulated
*                  data with this option SHOULD follow the same format as the outer-layer options, but is
*                  not mandatory.
*********************************************************************************************************
*/

static  void  DHCPc_PrintOptions (CPU_INT08U  *options)
{
    CPU_INT32U  opt;
    CPU_INT32U  optlen;
    CPU_INT32U  vsopt;
    CPU_INT32U  vsoptlen;
    CPU_INT32U  cookie;
    CPU_INT32U  safety;
    CPU_INT32U  i;


                                                                /* If DHCP_STD_MAGIC_COOKIE doesn't exist, then don't   */
                                                                /* process options...                                   */
    Mem_Copy(&cookie, options, 4);
    if (cookie != NET_UTIL_HOST_TO_NET_32(DHCP_STD_MAGIC_COOKIE)) {
        return;
    }
    options += 4;

    safety   = 0;
    while (*options  != 0xFF) {
        if (safety++  > 10000) {
            APP_TRACE_DEBUG(("Aborting, overflow likely.\n"));
            break;
        }
        opt = *options++;
        if (opt == 0) {                                         /* Padding.                                             */
            continue;
        }
        APP_TRACE_DEBUG(("        option %3d: ", opt));
        optlen  =  *options++;
        if (opt == DHCP_OPT_DHCP_MESSAGE_TYPE) {
            APP_TRACE_DEBUG(("DHCP%s", DHCPc_GetOptStr(*options++)));
        }
                                                                /* See Note (1).                                        */
        else if (opt == DHCP_OPT_VENDOR_SPECIFIC_INFORMATION) {
            APP_TRACE_DEBUG(("\n"));
            while(*options != 0xFF) {
                vsopt       = *options++;
                vsoptlen    = *options++;
                APP_TRACE_DEBUG(("        vso %3d: ",vsopt));
                APP_TRACE_DEBUG(("0x"));
                for(i = 0; i < vsoptlen; i++) {
                    APP_TRACE_DEBUG(("%02x", *options++));
                }
                if (*options != 0xFF) {
                    APP_TRACE_DEBUG(("\n"));
                }
            }
            options++;                                          /* Skip over the 0xff within this option subset.        */
        } else if ((opt <  DHCP_OPT_HOST_NAME)            ||
                   (opt == DHCP_OPT_BROADCAST_ADDRESS)    ||
                   (opt == DHCP_OPT_REQUESTED_IP_ADDRESS) ||
                   (opt == DHCP_OPT_SERVER_IDENTIFIER)    ||
                   (opt == DHCP_OPT_NETWORK_INFORMATION_SERVER)) {
            for (i = 0; i < optlen; i++) {
                APP_TRACE_DEBUG(("%d ", *options++));
            }
        } else if ((opt == DHCP_OPT_NETWORK_INFORMATION_SERVICE_DOMAIN) ||
                   (opt == DHCP_OPT_VENDOR_CLASS_IDENTIFIER)) {
            for (i = 0; i < optlen; i++) {
                APP_TRACE_DEBUG(("%c", *options++));
            }
        } else if (opt == DHCP_OPT_CLIENT_IDENTIFIER) {
            APP_TRACE_DEBUG(("%d 0x", *options++));
            for (i = 1; i < optlen; i++) {
                APP_TRACE_DEBUG(("%02x",*options++));
            }
        } else {
            APP_TRACE_DEBUG(("0x"));
            for (i = 0; i < optlen; i++) {
                APP_TRACE_DEBUG(("%02x", *options++));
            }
        }
        APP_TRACE_DEBUG(("\n"));
    }
}


/*
*********************************************************************************************************
*                                              DHCPc_GetOperStr()
*
* Description : DHCP packet printing function.
*********************************************************************************************************
*/

static  CPU_CHAR  *DHCPc_GetOperStr (CPU_INT32U  op)
{
    switch (op) {
        case DHCP_OP_REQUEST:
             return ("REQUEST");

        case DHCP_OP_REPLY:
             return ("REPLY");

        default:
             return ("???");
    }
}


/*
*********************************************************************************************************
*                                              DHCPc_GetOptStr()
*
* Description : DHCP packet printing function.
*********************************************************************************************************
*/

static  CPU_CHAR  *DHCPc_GetOptStr (CPU_INT32U  op)
{
    switch (op) {
        case DHCP_MSG_DISCOVER:
             return ("DISCOVER");

        case DHCP_MSG_OFFER:
             return ("OFFER");

        case DHCP_MSG_REQUEST:
             return ("REQUEST");

        case DHCP_MSG_DECLINE:
             return ("DECLINE");

        case DHCP_MSG_ACK:
             return ("ACK");

        case DHCP_MSG_NACK:
             return ("NACK");

        case DHCP_MSG_RELEASE:
             return ("RELEASE");

        default:
             return ("???");
    }
}
#endif  /* APP_DHCPc_EN */


#if APP_FTPs_EN
/*
*********************************************************************************************************
*                                     FTPs AUTHENTICATION CALLBACK
*********************************************************************************************************
*/

CPU_BOOLEAN  FTPs_AuthUser (FTPs_SESSION_STRUCT  *ftp_session)
{
                                                                /* This authentication is trivial, accepts              */
                                                                /* everyone sets BasePath and RelPath to ROOT           */
                                                                /* allowing access to the whole filesystem.  This       */
                                                                /* MUST be modified according to your security          */
                                                                /* policies.                                            */

                                                                /* Initialize BasePath to ROOT.                         */
    Str_FmtPrint((char *)ftp_session->BasePath,
        sizeof(ftp_session->BasePath),
       "%c",
        FTPs_PATH_SEP_CHAR);

                                                                /* Initialize RelPath to ROOT.                          */
    Str_FmtPrint((char *)ftp_session->RelPath,
        sizeof(ftp_session->RelPath),
       "%c",
        FTPs_PATH_SEP_CHAR);

    return (DEF_OK);
}
#endif  /* APP_FTPs_EN */


#if APP_HTTPs_EN
/*
*********************************************************************************************************
*                                          HTTPs VALUE REQUEST
*********************************************************************************************************
*/

CPU_BOOLEAN  HTTPs_ValReq (CPU_CHAR     *Variable,
                           CPU_CHAR    **Val)
{
    static  CPU_CHAR  buf[20];
            CPU_FP32  ver;
            CPU_FP32  time;
            NET_ERR   err;


    Str_Copy(&buf[0], "%%%%%%%%");
   *Val = &buf[0];

                                                                        /* ----------------- OS VALS ------------------ */
    if (       Str_Cmp(Variable, "OS_VERSION"            ) == 0) {
        ver = (CPU_FP32)OS_VERSION / 100;
        Str_FmtNbr_32(ver,                                       2,  2, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "OS_NBR_TASKS"          ) == 0) {
        Str_FmtNbr_32(OSTaskCtr,                                 2,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "OS_CPU_USAGE"          ) == 0) {
        Str_FmtNbr_32(OSCPUUsage,                                3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "OS_CPU_USAGE_MAX"      ) == 0) {
        Str_FmtNbr_32(OS_CPU_UsageMax,                           3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "OS_TIME"               ) == 0) {
        time = (CPU_FP32)OSTime / DEF_TIME_NBR_mS_PER_SEC;
        Str_FmtNbr_32(time,                                      7,  3, DEF_NO, DEF_YES, &buf[0]);

                                                                        /* ----------------- NET VALS ----------------- */
    } else if (Str_Cmp(Variable, "NET_VERSION"           ) == 0) {
        ver = (CPU_FP32)NET_VERSION / 100;
        Str_FmtNbr_32(ver,                                       2,  2, DEF_NO, DEF_YES, &buf[0]);

                                                                        /* ---------------- ADDR VALS ----------------- */
    } else if (Str_Cmp(Variable, "MAC"                   ) == 0) {
        NetASCII_MAC_to_Str(&NetIF_MAC_Addr[0], &buf[0], DEF_NO, &err);

    } else if (Str_Cmp(Variable, "IP"                    ) == 0) {
        NetASCII_IP_to_Str(App_IP_Addr,         &buf[0], DEF_NO, &err);

    } else if (Str_Cmp(Variable, "MASK"                  ) == 0) {
        NetASCII_IP_to_Str(App_IP_Mask,         &buf[0], DEF_NO, &err);

    } else if (Str_Cmp(Variable, "GW"                    ) == 0) {
        NetASCII_IP_to_Str(App_IP_DfltGateway,  &buf[0], DEF_NO, &err);

                                                                        /* ---------------- CONN VALS ----------------- */
    } else if (Str_Cmp(Variable, "NBR_SOCK"              ) == 0) {
        Str_FmtNbr_32(NET_SOCK_CFG_NBR_SOCK,                     3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_SOCK_AVAIL"        ) == 0) {
        Str_FmtNbr_32(NetSock_PoolStat.EntriesAvail,             3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_SOCK_USED"         ) == 0) {
        Str_FmtNbr_32(NetSock_PoolStat.EntriesUsed,              3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_SOCK_USED_MAX"     ) == 0) {
        Str_FmtNbr_32(NetSock_PoolStat.EntriesUsedMax,           3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_SOCK_USED_TOT"     ) == 0) {
        Str_FmtNbr_32(NetSock_PoolStat.EntriesAllocatedCtr,      5,  0, DEF_NO, DEF_YES, &buf[0]);


    } else if (Str_Cmp(Variable, "NBR_TCP_CONN"          ) == 0) {
        Str_FmtNbr_32(NET_TCP_CFG_NBR_CONN,                      3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_TCP_CONN_AVAIL"    ) == 0) {
        Str_FmtNbr_32(NetTCP_ConnPoolStat.EntriesAvail,          3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_TCP_CONN_USED"     ) == 0) {
        Str_FmtNbr_32(NetTCP_ConnPoolStat.EntriesUsed,           3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_TCP_CONN_USED_MAX" ) == 0) {
        Str_FmtNbr_32(NetTCP_ConnPoolStat.EntriesUsedMax,        3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_TCP_CONN_USED_TOT" ) == 0) {
        Str_FmtNbr_32(NetTCP_ConnPoolStat.EntriesAllocatedCtr,   5,  0, DEF_NO, DEF_YES, &buf[0]);


    } else if (Str_Cmp(Variable, "NBR_CONN"              ) == 0) {
        Str_FmtNbr_32(NET_CONN_CFG_NBR_CONN,                     3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_CONN_AVAIL"        ) == 0) {
        Str_FmtNbr_32(NetConn_PoolStat.EntriesAvail,             3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_CONN_USED"         ) == 0) {
        Str_FmtNbr_32(NetConn_PoolStat.EntriesUsed,              3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_CONN_USED_MAX"     ) == 0) {
        Str_FmtNbr_32(NetConn_PoolStat.EntriesUsedMax,           3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_CONN_USED_TOT"     ) == 0) {
        Str_FmtNbr_32(NetConn_PoolStat.EntriesAllocatedCtr,      5,  0, DEF_NO, DEF_YES, &buf[0]);


                                                                        /* ---------------- BUF VALS ------------------ */
    } else if (Str_Cmp(Variable, "NBR_BUF_SMALL"         ) == 0) {
        Str_FmtNbr_32(NET_BUF_CFG_NBR_SMALL,                     3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_BUF_SMALL_AVAIL"   ) == 0) {
        Str_FmtNbr_32(NetBuf_SmallPoolStat.EntriesAvail,         3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_BUF_SMALL_USED"    ) == 0) {
        Str_FmtNbr_32(NetBuf_SmallPoolStat.EntriesUsed,          3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_BUF_SMALL_USED_MAX") == 0) {
        Str_FmtNbr_32(NetBuf_SmallPoolStat.EntriesUsedMax,       3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_BUF_SMALL_USED_TOT") == 0) {
        Str_FmtNbr_32(NetBuf_SmallPoolStat.EntriesAllocatedCtr,  6,  0, DEF_NO, DEF_YES, &buf[0]);


    } else if (Str_Cmp(Variable, "NBR_BUF_LARGE"         ) == 0) {
        Str_FmtNbr_32(NET_BUF_CFG_NBR_LARGE,                     3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_BUF_LARGE_AVAIL"   ) == 0) {
        Str_FmtNbr_32(NetBuf_LargePoolStat.EntriesAvail,         3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_BUF_LARGE_USED"    ) == 0) {
        Str_FmtNbr_32(NetBuf_LargePoolStat.EntriesUsed,          3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_BUF_LARGE_USED_MAX") == 0) {
        Str_FmtNbr_32(NetBuf_LargePoolStat.EntriesUsedMax,       3,  0, DEF_NO, DEF_YES, &buf[0]);

    } else if (Str_Cmp(Variable, "NBR_BUF_LARGE_USED_TOT") == 0) {
        Str_FmtNbr_32(NetBuf_LargePoolStat.EntriesAllocatedCtr,  6,  0, DEF_NO, DEF_YES, &buf[0]);

    }

    HTTPs_TRACE_INFO(("HTTP-s: VALUE REQUESTED: %s = %s.\n", Variable, *Val));

    return (DEF_OK);
}


/*
*********************************************************************************************************
*                                         HTTPs VALUE RECEIVED
*********************************************************************************************************
*/

CPU_BOOLEAN  HTTPs_ValRx (CPU_CHAR  *Variable,
                          CPU_CHAR  *Val)
{
    HTTPs_TRACE_INFO(("HTTP-s: VALUE RECEIVED: %s = %s.\n", Variable, Val));
    return (DEF_OK);
}
#endif  /* APP_HTTPs_EN */
