/*
*********************************************************************************************************
*
*                                             EXAMPLE CODE
*
*                          (c) Copyright 2003-2007; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*
*               Knowledge of the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                          CONFIGURATION FILE
*
* Filename      : app_cfg.h
* Version       : V1.89
* Programmer(s) : Jean-Denis Hatier
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                        MODULE ENABLE / DISABLE
*********************************************************************************************************
*/

#define  APP_TCPIP_EN                           DEF_ENABLED
#define  APP_VIEW_EN                            OS_VIEW_MODULE

#define  APP_CLK_EN                             DEF_DISABLED
#define  APP_DHCPc_EN                           DEF_DISABLED
#define  APP_DNSc_EN                            DEF_DISABLED
#define  APP_FS_EN                              DEF_DISABLED
#define  APP_FTPc_EN                            DEF_DISABLED
#define  APP_FTPs_EN                            DEF_DISABLED
#define  APP_HTTPs_EN                           DEF_DISABLED
#define  APP_POP3c_EN                           DEF_DISABLED
#define  APP_SMTPc_EN                           DEF_DISABLED
#define  APP_SNTPc_EN                           DEF_DISABLED
#define  APP_TFTPs_EN                           DEF_DISABLED
#define  APP_TTCP_EN                            DEF_ENABLED


/*
*********************************************************************************************************
*                                              TASKS NAMES
*********************************************************************************************************
*/

#define  CLK_OS_CFG_TASK_NAME                  "RealTime Clock"
#define  DHCPc_OS_CFG_TASK_NAME                "DHCP (Client)"
#define  FTPs_SRV_OS_CFG_TASK_NAME             "FTP (Server)"
#define  FTPs_CTRL_OS_CFG_TASK_NAME            "FTP (Control)"
#define  HTTPs_OS_CFG_TASK_NAME                "HTTP (Server)"
#define  TFTPs_OS_CFG_TASK_NAME                "TFTP (Server)"
#define  TTCP_OS_CFG_TASK_NAME                 "TTCP"


/*
*********************************************************************************************************
*                                           TASKS PRIORITIES
*********************************************************************************************************
*/

#define  CLK_OS_CFG_TASK_PRIO                              4
#define  APP_START_OS_CFG_TASK_PRIO                        5

#define  NET_OS_CFG_IF_RX_TASK_PRIO                        8
#define  NET_OS_CFG_TMR_TASK_PRIO                          9

#define  TTCP_OS_CFG_TASK_PRIO                            16
#define  HTTPs_OS_CFG_TASK_PRIO                           17
#define  FTPs_SRV_OS_CFG_TASK_PRIO                        18
#define  FTPs_CTRL_OS_CFG_TASK_PRIO                       19
#define  TFTPs_OS_CFG_TASK_PRIO                           20
#define  DHCPc_OS_CFG_TASK_PRIO                           21

#define  APP_1_OS_CFG_TASK_PRIO                           32
#define  OS_TASK_TMR_PRIO                (OS_LOWEST_PRIO - 2)


/*
*********************************************************************************************************
*                                              STACK SIZES
*                             Size of the task stacks (# of OS_STK entries)
*********************************************************************************************************
*/

#define  CLK_OS_CFG_TASK_STK_SIZE                        512
#define  APP_START_OS_CFG_TASK_STK_SIZE                 1024

#define  NET_OS_CFG_IF_RX_TASK_STK_SIZE                 1024
#define  NET_OS_CFG_TMR_TASK_STK_SIZE                   1024

#define  TTCP_OS_CFG_TASK_STK_SIZE                      1024
#define  HTTPs_OS_CFG_TASK_STK_SIZE                     2048
#define  FTPs_SRV_OS_CFG_TASK_STK_SIZE                  1024
#define  FTPs_CTRL_OS_CFG_TASK_STK_SIZE                 4096
#define  TFTPs_OS_CFG_TASK_STK_SIZE                      512
#define  DHCPc_OS_CFG_TASK_STK_SIZE                      256

#define  APP_1_OS_CFG_TASK_STK_SIZE                      512


/*
*********************************************************************************************************
*                                                  LIB
*********************************************************************************************************
*/

#define  uC_CFG_OPTIMIZE_ASM_EN                 DEF_ENABLED
#define  LIB_STR_CFG_FP_EN                      DEF_ENABLED


/*
*********************************************************************************************************
*                                           CLK CONFIGURATION
*********************************************************************************************************
*/

#define  CLK_CFG_SIGNAL_RATE                    OS_TICKS_PER_SEC/* Number of time ClkTask() is called per second.       */


#define  CLK_CFG_ARG_CHK_EN                     DEF_ENABLED     /* Enable / disable argument checking.                  */

#define  CLK_CFG_SIGNAL_EN                      DEF_DISABLED    /* Enabled:  clock source is 'clock tick' ISR mode.     */
                                                                /* Disabled: clock source is kernel task time delay.    */

#define  CLK_CFG_STR_CONV_EN                    DEF_ENABLED     /* Enable / disable clock conversion to string.         */


/*
*********************************************************************************************************
*                                                  NIC
*********************************************************************************************************
*/

                                                                /* AT91RM9200 EMAC MAC address source :                 */
                                                                /* AT91RM9200_EMAC_MAC_ADDR_SEL_EEPROM from NIC's EEPROM*/
                                                                /* AT91RM9200_EMAC_MAC_ADDR_SEL_CFG    from application.*/
#define  AT91RM9200_EMAC_CFG_MAC_ADDR_SEL       AT91RM9200_EMAC_MAC_ADDR_SEL_CFG


/*
*********************************************************************************************************
*                                    NO DHCP / STATIC CONFIGURATION
*********************************************************************************************************
*/

#define  APP_CFG_IP_ADDR_STR_THIS_HOST         "192.168.1.103"
#define  APP_CFG_IP_ADDR_STR_NET_MASK          "255.255.255.0"
#define  APP_CFG_IP_ADDR_STR_DFLT_GATEWAY      "192.168.0.1"
#define  APP_CFG_IP_ADDR_STR_DNS_SRVR          "192.168.0.2"
#define  APP_CFG_IP_ADDR_STR_NTP_SRVR          "132.246.168.164"
#define  APP_CFG_CLK_UTC_OFFSET                 -18000
#define  APP_CFG_DFLT_HOST_NAME                "CSB637"


/*
*********************************************************************************************************
*                                                 DHCPc
*********************************************************************************************************
*/

                                                                /* DHCP server IP port. Default is 67.                  */
#define  DHCPc_CFG_IP_PORT_SERVER                         67
                                                                /* DHCP client IP port. Default is 68.                  */
#define  DHCPc_CFG_IP_PORT_CLIENT                         68

                                                                /* Maximum number of attempts to negotiate lease with   */
                                                                /* the DHCP server.                                     */
#define  DHCPc_CFG_MAX_REQ_LEASE_RETRY                     5

                                                                /* Once the DHCP server has assigned the client an      */
                                                                /* address, the later may perform a final check prior   */
                                                                /* to use this address in order to make sure it is not  */
                                                                /* being used by another host on the network.           */
#define  DHCPc_CFG_ADDR_VALIDATE_EN             DEF_ENABLED

                                                                /* Number of seconds to wait for a possible ARP reply   */
                                                                /* when performing the final check on the suggested     */
                                                                /* address.  The minimum is 1 second.                   */
#define  DHCPc_CFG_ADDR_VALIDATE_WAIT_TIME_MS           3000

                                                                /* The broadcast bit is used to instruct the server to  */
                                                                /* use broadcast packets instead of unicast.            */
                                                                /* Useful when the IP stack cannot receive unicast      */
                                                                /* packets when not fully configured.                   */
                                                                /* This is our case, so the value MUST be DEF_ENABLED.  */
#define  DHCPc_CFG_BROADCAST_BIT                DEF_ENABLED

                                                                /* Size of array used for the client identifier,        */
                                                                /* typically a 48-bit MAC/net address.                  */
                                                                /* The minimum size value is 2.                         */
#define  DHCPc_CFG_CLIENT_ID_SIZE               NET_IF_ADDR_SIZE

                                                                /* Size of array used for the vendor class identifier.  */
                                                                /* Set this value according to the argument length      */
                                                                /* passed to the DHCPc_SetVendorClassID function.       */
                                                                /* The minimum size value is 1.                         */
#define  DHCPc_CFG_VENDOR_CLASS_ID_SIZE                   11

                                                                /* Size of the array used for the requested parameter   */
                                                                /* list.  Defines the maximum number of parameters to   */
                                                                /* request when invoking DHCPc_SetParamReqList.         */
#define  DHCPc_CFG_PARAM_REQ_LIST_SIZE                    10


/*
*********************************************************************************************************
*                                                 DNSc
*********************************************************************************************************
*/

#define  DNSc_CFG_IPPORT                                  53    /* Client IP port. Default is 53.                       */

#define  DNSc_CFG_MAX_RX_TIMEOUT_MS                     5000    /* Maximum inactivity time (ms) on RX.                  */
#define  DNSc_CFG_MAX_TX_TIMEOUT_MS                     5000    /* Maximum inactivity time (ms) on TX.                  */


/*
*********************************************************************************************************
*                                                 FTPc
*********************************************************************************************************
*/

#define  FTPc_CFG_CTRL_IPPORT                             21    /* FTP Control IP port. Default is 21.                  */
#define  FTPc_CFG_DTP_IPPORT                              20    /* FTP Data IP port.    Default is 20.                  */

#define  FTPc_CFG_CTRL_MAX_CONN_TIMEOUT_MS             30000    /* Maximum inactivity time (ms) on CONNECT.             */
#define  FTPc_CFG_CTRL_MAX_RX_TIMEOUT_MS               30000    /* Maximum inactivity time (ms) on RX.                  */
#define  FTPc_CFG_CTRL_MAX_TX_TIMEOUT_MS               30000    /* Maximum inactivity time (ms) on TX.                  */

#define  FTPc_CFG_DTP_MAX_ACCEPT_TIMEOUT_MS            30000    /* Maximum inactivity time (ms) on ACCEPT.              */
#define  FTPc_CFG_DTP_MAX_CONN_TIMEOUT_MS              30000    /* Maximum inactivity time (ms) on CONNECT.             */
#define  FTPc_CFG_DTP_MAX_RX_TIMEOUT_MS                30000    /* Maximum inactivity time (ms) on RX.                  */
#define  FTPc_CFG_DTP_MAX_TX_TIMEOUT_MS                30000    /* Maximum inactivity time (ms) on TX.                  */

#define  FTPc_CFG_CTRL_MAX_CONN_RETRY                      3    /* Maximum number of retries on CONNECT.                */
#define  FTPc_CFG_CTRL_MAX_RX_RETRY                        3    /* Maximum number of retries on RX.                     */
#define  FTPc_CFG_CTRL_MAX_TX_RETRY                        3    /* Maximum number of retries on TX.                     */

#define  FTPc_CFG_DTP_MAX_ACCEPT_RETRY                     3    /* Maximum number of retries on ACCEPT.                 */
#define  FTPc_CFG_DTP_MAX_CONN_RETRY                       3    /* Maximum number of retries on CONNECT.                */
#define  FTPc_CFG_DTP_MAX_RX_RETRY                         3    /* Maximum number of retries on RX.                     */
#define  FTPc_CFG_DTP_MAX_TX_RETRY                         3    /* Maximum number of retries on TX.                     */

#define  FTPc_CFG_MAX_USER_LEN                            32    /* Maximum length for user name.                        */
#define  FTPc_CFG_MAX_PASS_LEN                            32    /* Maximum length for password.                         */

#define  FTPc_CFG_USE_FS                                   1    /* If 1, interface functions using FS are enabled.      */
                                                                /* If 0, only functions using RAM buffers are enabled.  */


/*
*********************************************************************************************************
*                                                 FTPs
*********************************************************************************************************
*/

#define  FTPs_CFG_CTRL_IPPORT                             21    /* FTP Control IP port. Default is 21.                  */
#define  FTPs_CFG_DTP_IPPORT                              20    /* FTP Data IP port.    Default is 20.                  */

#define  FTPs_CFG_CTRL_MAX_ACCEPT_TIMEOUT_MS              -1    /* Maximum inactivity time (ms) on ACCEPT.              */
#define  FTPs_CFG_CTRL_MAX_RX_TIMEOUT_MS               30000    /* Maximum inactivity time (ms) on RX.                  */
#define  FTPs_CFG_CTRL_MAX_TX_TIMEOUT_MS               30000    /* Maximum inactivity time (ms) on TX.                  */

#define  FTPs_CFG_DTP_MAX_ACCEPT_TIMEOUT_MS            30000    /* Maximum inactivity time (ms) on ACCEPT.              */
#define  FTPs_CFG_DTP_MAX_CONN_TIMEOUT_MS              30000    /* Maximum inactivity time (ms) on CONNECT.             */
#define  FTPs_CFG_DTP_MAX_RX_TIMEOUT_MS                30000    /* Maximum inactivity time (ms) on RX.                  */
#define  FTPs_CFG_DTP_MAX_TX_TIMEOUT_MS                30000    /* Maximum inactivity time (ms) on TX.                  */

#define  FTPs_CFG_CTRL_MAX_ACCEPT_RETRY                   -1    /* Maximum number of retries on ACCEPT.                 */
#define  FTPs_CFG_CTRL_MAX_RX_RETRY                        3    /* Maximum number of retries on RX.                     */
#define  FTPs_CFG_CTRL_MAX_TX_RETRY                        3    /* Maximum number of retries on TX.                     */

#define  FTPs_CFG_DTP_MAX_ACCEPT_RETRY                     3    /* Maximum number of retries on ACCEPT.                 */
#define  FTPs_CFG_DTP_MAX_CONN_RETRY                       3    /* Maximum number of retries on CONNECT.                */
#define  FTPs_CFG_DTP_MAX_RX_RETRY                         3    /* Maximum number of retries on RX.                     */
#define  FTPs_CFG_DTP_MAX_TX_RETRY                         3    /* Maximum number of retries on TX.                     */

#define  FTPs_CFG_MAX_USER_LEN                            32    /* Maximum length for user name.                        */
#define  FTPs_CFG_MAX_PASS_LEN                            32    /* Maximum length for password.                         */


/*
*********************************************************************************************************
*                                                 HTTPs
*********************************************************************************************************
*/

#define  HTTPs_CFG_IPPORT                                 80    /* HTTP server IP port. Default is 80.                  */

#define  HTTPs_CFG_MAX_ACCEPT_TIMEOUT_MS                  -1    /* Maximum inactivity time (ms) on ACCEPT.              */
#define  HTTPs_CFG_MAX_RX_TIMEOUT_MS                   30000    /* Maximum inactivity time (ms) on RX.                  */
#define  HTTPs_CFG_MAX_TX_TIMEOUT_MS                   30000    /* Maximum inactivity time (ms) on TX.                  */

#define  HTTPs_CFG_MAX_ACCEPT_RETRY                       -1    /* Maximum number of retries on ACCEPT.                 */
#define  HTTPs_CFG_MAX_RX_RETRY                            3    /* Maximum number of retries on RX.                     */
#define  HTTPs_CFG_MAX_TX_RETRY                            3    /* Maximum number of retries on TX.                     */

#define  HTTPs_CFG_FILE_RD_BUF_LEN                      1024    /* Length of buffer used to read file.                  */
#define  HTTPs_CFG_FILE_TX_BUF_LEN                      1024    /* Length of buffer used to send file.                  */

#define  HTTPs_CFG_TOK_PARSE_EN                 DEF_ENABLED     /* Enable / disable token parsing (${}).                */
                                                                /* Size of parse table.  Set to "1" if                  */
                                                                /* HTTPs_CFG_TOKEN_PARSE_EN is disabled in order to     */
#define  HTTPs_CFG_TOK_PARSE_TBL_SIZE                    256    /* minimize memory consumption.                         */

                                                                /* Root path for HTTP documents in filesystem.          */
#define  HTTPs_CFG_FS_ROOT                     "/HTTPRoot"      /* Filesystem-specific symbols can be used.             */
#define  HTTPs_CFG_DFLT_FILE                   "index.htm"      /* Default file to load if no filename specified in URL.*/

                                                                /* Default HTML document returned when the requested    */
                                                                /* HTML document is not found (HTTP error #404).        */
#define  HTTPs_CFG_ERR_MSG_HTML_NOT_FOUND      "<HTML>\r\n" \
                                               "<BODY>\r\n" \
                                               "<HEAD><TITLE>SYSTEM ERROR</TITLE></HEAD>\r\n" \
                                               "<H1>NOT FOUND</H1>\r\n" \
                                               "The requested object does not exist on this server.\r\n" \
                                               "</BODY>\r\n" \
                                               "</HTML>\r\n"


/*
*********************************************************************************************************
*                                                 POP3c
*********************************************************************************************************
*/

#define  POP3c_CFG_IPPORT                                110    /* POP3 client IP port. Default is 110. See note #1.    */

#define  POP3c_CFG_MAX_CONN_TIMEOUT_MS                 30000    /* Maximum inactivity time (ms) on CONNECT.             */
#define  POP3c_CFG_MAX_RX_TIMEOUT_MS                   30000    /* Maximum inactivity time (ms) on RX.                  */
#define  POP3c_CFG_MAX_TX_TIMEOUT_MS                   30000    /* Maximum inactivity time (ms) on TX.                  */

#define  POP3c_CFG_MAX_CONN_RETRY                          3    /* Maximum number of retries on CONNECT.                */
#define  POP3c_CFG_MAX_RX_RETRY                            3    /* Maximum number of retries on RX.                     */
#define  POP3c_CFG_MAX_TX_RETRY                            3    /* Maximum number of retries on TX.                     */


/*
*********************************************************************************************************
*                                                 SMTPc
*********************************************************************************************************
*/

#define  SMTPc_CFG_IPPORT                                 25    /* SMTP client IP port. Default is 25.                  */

#define  SMTPc_CFG_MAX_CONN_TIMEOUT_MS                 30000    /* Maximum inactivity time (ms) on CONNECT.             */
#define  SMTPc_CFG_MAX_RX_TIMEOUT_MS                   30000    /* Maximum inactivity time (ms) on RX.                  */
#define  SMTPc_CFG_MAX_TX_TIMEOUT_MS                   30000    /* Maximum inactivity time (ms) on TX.                  */

#define  SMTPc_CFG_MAX_CONN_RETRY                          3    /* Maximum number of retries on CONNECT.                */
#define  SMTPc_CFG_MAX_RX_RETRY                            3    /* Maximum number of retries on RX.                     */
#define  SMTPc_CFG_MAX_TX_RETRY                            3    /* Maximum number of retries on TX.                     */

#define  SMTPc_CFG_MBOX_NAME_DISP_LEN                     50    /* Maximum length of sender's name.                     */
#define  SMTPc_CFG_MSG_SUBJECT_LEN                        50    /* Maximum length of message subject.                   */

#define  SMTPc_CFG_MSG_MAX_TO                              5    /* Maximum number of TO recipients.                     */
#define  SMTPc_CFG_MSG_MAX_CC                              5    /* Maximum number of CC  recipients.                    */
#define  SMTPc_CFG_MSG_MAX_BCC                             5    /* Maximum number of BCC recipients.                    */
#define  SMTPc_CFG_MSG_MAX_ATTACH                          5    /* Maximum number of message attachments.               */


/*
*********************************************************************************************************
*                                                 SNTPc
*********************************************************************************************************
*/

#define  SNTPc_CFG_IPPORT                                123    /* SNTP client IP port. Default is 123.                 */

#define  SNTPc_CFG_MAX_RX_TIMEOUT_MS                    5000    /* Maximum inactivity time (ms) on RX.                  */
#define  SNTPc_CFG_MAX_TX_TIMEOUT_MS                    5000    /* Maximum inactivity time (ms) on TX.                  */


/*
*********************************************************************************************************
*                                                 TFTPc
*********************************************************************************************************
*/

#define  TFTPc_CFG_IPPORT                                 69    /* TFTP client IP port. Default is 69.                  */
#define  TFTPc_DFLT_PORT                                  69    /* TFTP client IP port. Default is 69. Deprecated.      */

#define  TFTPc_CFG_MAX_RX_TIMEOUT_MS                    5000    /* Maximum inactivity time (ms) on RX.                  */
#define  TFTPc_CFG_MAX_TX_TIMEOUT_MS                    5000    /* Maximum inactivity time (ms) on TX.                  */


/*
*********************************************************************************************************
*                                                 TFTPs
*********************************************************************************************************
*/

#define  TFTPs_CFG_IPPORT                                 69    /* TFTP server IP port. Default is 69.                  */

#define  TFTPs_CFG_MAX_RX_TIMEOUT_MS                    5000    /* Maximum inactivity time (ms) on RX.                  */
#define  TFTPs_CFG_MAX_TX_TIMEOUT_MS                    5000    /* Maximum inactivity time (ms) on TX.                  */

#define  TFTPs_TRACE_HIST_SIZE                            16    /* TFTP server trace system history size.               */
#define  TFTPs_TRACE_STR_SIZE                             80    /* TFTP server trace system string size.                */


/*
*********************************************************************************************************
*                                                 TTCP
*********************************************************************************************************
*/

#define  TTCP_CFG_MAX_ACCEPT_TIMEOUT_MS                30000    /* Maximum inactivity time (ms) on ACCEPT.              */
#define  TTCP_CFG_MAX_CONN_TIMEOUT_MS                  30000    /* Maximum inactivity time (ms) on CONNECT.             */
#define  TTCP_CFG_MAX_RX_TIMEOUT_MS                    30000    /* Maximum inactivity time (ms) on RX.                  */
#define  TTCP_CFG_MAX_TX_TIMEOUT_MS                    30000    /* Maximum inactivity time (ms) on TX.                  */

#define  TTCP_CFG_MAX_ACCEPT_RETRY                         3    /* Maximum number of retries on ACCEPT.                 */
#define  TTCP_CFG_MAX_CONN_RETRY                           3    /* Maximum number of retries on CONNECT.                */
#define  TTCP_CFG_MAX_RX_RETRY                             3    /* Maximum number of retries on RX.                     */
#define  TTCP_CFG_MAX_TX_RETRY                             3    /* Maximum number of retries on TX.                     */

/*
*********************************************************************************************************
*                                                TRACING
*********************************************************************************************************
*/

#include  <monlib.h>

#define  TRACE_LEVEL_OFF                                   0
#define  TRACE_LEVEL_INFO                                  1
#define  TRACE_LEVEL_DEBUG                                 2

#define  APP_TRACE_LEVEL                        TRACE_LEVEL_DEBUG
#define  APP_TRACE                              mon_printf

#define  CLK_TRACE_LEVEL                        TRACE_LEVEL_OFF
#define  CLK_TRACE                              mon_printf

#define  DHCPc_TRACE_LEVEL                      TRACE_LEVEL_OFF
#define  DHCPc_TRACE                            mon_printf

#define  DNSc_TRACE_LEVEL                       TRACE_LEVEL_OFF
#define  DNSc_TRACE                             mon_printf

#define  FTPc_TRACE_LEVEL                       TRACE_LEVEL_OFF
#define  FTPc_TRACE                             mon_printf

#define  FTPs_TRACE_LEVEL                       TRACE_LEVEL_OFF
#define  FTPs_TRACE                             mon_printf

#define  HTTPs_TRACE_LEVEL                      TRACE_LEVEL_OFF
#define  HTTPs_TRACE                            mon_printf

#define  POP3c_TRACE_LEVEL                      TRACE_LEVEL_OFF
#define  POP3c_TRACE                            mon_printf

#define  SMTPc_TRACE_LEVEL                      TRACE_LEVEL_OFF
#define  SMTPc_TRACE                            mon_printf

#define  SNTPc_TRACE_LEVEL                      TRACE_LEVEL_OFF
#define  SNTPc_TRACE                            mon_printf

#define  TFTPs_TRACE_LEVEL                      TRACE_LEVEL_OFF
#define  TFTPs_TRACE                            mon_printf

#define  TTCP_TRACE_LEVEL                       TRACE_LEVEL_INFO
#define  TTCP_TRACE                             mon_printf

#define  APP_TRACE_INFO(x)                    ((APP_TRACE_LEVEL >= TRACE_LEVEL_INFO)  ? (void)(APP_TRACE x) : (void)0)
#define  APP_TRACE_DEBUG(x)                   ((APP_TRACE_LEVEL >= TRACE_LEVEL_DEBUG) ? (void)(APP_TRACE x) : (void)0)
